%% EXAMPLE HYPER-A PROCESSING SCRIPT
%
% This script will process raw Hyper-a data to absorption coefficient
% (1/m). Data can be imported from Hyper-a .bin file or .mat files. This
% script calls the core Hyper-a processing functions in 'Hypera_lib.m'.
%
% INPUTS
% Edit the first 'Load Data' section to point to the data files (sample and
% pure water reference). Both temperature (Celsius) and salinity (PSU) must
% be done and entered below. Lastly, you must specify the to the
% instrument specific calibration file (.mat file).
%
% OUTPUT
% See 'hypera_proc' table for final absorption results. The absorption
% coefficient (1/m) in the table has absorption by water removed.
% Wavelengths and instrument configuration information are stored in the 
% table properties (hypera_proc.Properties.CustomProperties). Other
% variables contain intermediate processing results and can be understood
% by studying the code and comments below.
%
% Version 1.1
% Sequoia Scientific, Inc. - 8/1/2024
%

%% Load Data
 sample = load('.\ExampleData\Sample.mat'); % data collected in .mat file in real time using 'Hypera_RealTimeExample.m'
% OR, data could be loaded from a file using Hypera_ReadBin.m, example:
 %[sample.config, sample.dataTable] = Hypera_ReadBin(filename);

 purewater = load('.\ExampleData\PureWater.mat'); % data collected in .mat file in real time using 'Hypera_RealTimeExample.m'
% OR, data could be loaded from a file using Hypera_ReadBin.m, example:
% [purewater.config, purewater.dataTable] = Hypera_ReadBin(filename);

% Temperature and salinity values for the sample and pure water
% measurements
T_PureWater = 22;
S_PureWater = 0;
T_Sample = 22;
S_Sample = 0;

% path to calibration file supplied with instrument
load('.\ExampleData\SN8021_CAL_20240731.mat');

%% Subtract Dark and Normalize by Reference

sample_DarkRefCorrected    = Hypera_lib.CorrectSpectrum(cal, sample.dataTable, sample.config);
purewater_DarkRefCorrected = Hypera_lib.CorrectSpectrum(cal, purewater.dataTable, purewater.config);

%% Compute Hyper-a Absorption

% pure water is used as reference, compute theoretical pure water absorption spectrum
aW_ref = Hypera_lib.Get_IOCCG_aW(cal.wl,T_PureWater,S_PureWater)';

% compute hyper-a measured absorption using pure water as reference
a_tot_hypera = Hypera_lib.ComputeAbsorption(cal, ...
                                            aW_ref, ...
                                            purewater_DarkRefCorrected.NoFilter, ...
                                            sample_DarkRefCorrected.NoFilter);

%% Remove Pure Water Absorption From Hypera Result

aW_sample = Hypera_lib.Get_IOCCG_aW(cal.wl,T_Sample,S_Sample)';
a_hypera = a_tot_hypera - repmat(aW_sample,height(a_tot_hypera),1); 

%% Interpolate Over Saturated Pixels

saturation = isnan(a_hypera);
if any(any(saturation))
    for i_meas = 1:height(a_hypera)
       sat_wls = cal.wl(saturation(i_meas,:));
       if ~isempty(sat_wls)
        disp(['Warning: Interpolating over saturation in measurement ' num2str(i_meas) ' at wavelengths ' num2str(sat_wls')]); 
       end
    end
    a_hypera = fillmissing(a_hypera,'linear',2);
end

%% Merge results

if iscell(sample.dataTable)
    sample_data = vertcat(sample.dataTable{:});
else
    sample_data = sample.dataTable;
end

hypera_proc = sample_data(sample_data.recordID == 10, 2:7);
hypera_proc = addvars(hypera_proc,a_hypera,'NewVariableNames','absorption');

% save wavelength vector in the table properties
hypera_proc = addprop(hypera_proc,{'wavelengths'}, {'table'});
hypera_proc.Properties.CustomProperties.wavelengths = cal.wl;

% save instrument configuration in the table properties
hypera_proc = addprop(hypera_proc,{'config'}, {'table'});
if iscell(sample.config)
    config = sample.config{1};
else
    config = sample.config;
end
hypera_proc.Properties.CustomProperties.config = config;

%% plot results

figure
subplot(2,2,1)
plot(hypera_proc.Properties.CustomProperties.wavelengths,hypera_proc.absorption,'.-')
xlabel('Wavelength (nm)')
ylabel('Absorption (1/m)')
grid on

subplot(2,2,2)
plot(hypera_proc.date,hypera_proc.depth,'.-k', 'markersize', 10, 'linewidth', 1)
xlabel('Date')
ylabel('Depth (m)')
grid on

subplot(2,2,3)
plot(hypera_proc.date,hypera_proc.waterTemp,'.-k', 'markersize', 10, 'linewidth', 1)
xlabel('Date')
ylabel('Water Temperature (C)')
grid on

subplot(2,2,4)
plot(hypera_proc.date,hypera_proc.inputVoltage,'.-k', 'markersize', 10, 'linewidth', 1)
xlabel('Date')
ylabel('Input Voltage (V)')
grid on

