%% Hyper-a Library of Function
% Core functions used for processing Hyper-a data. Computation of
% absorption follows procedure described in:
%
% Röttgers, R., W. Schönfeld, P.-R. Kipp, and R. Doerffer, 2005: Practical test of a point-source integrating
% cavity absorption meter: the performance of different collector assemblies. Appl. Opt., 44: 5549–5560.
%
% Version 1.1
% Sequoia Scientific, Inc. - 8/1/2024
%
classdef Hypera_lib
    methods(Static)
        
        %% Remove dark and computes sig/ref for Hyper-a spectrometer data
        
        function DarkRefCorrected = CorrectSpectrum(cal, dataTable, config)
            
            if ~iscell(dataTable)
                % group data by record ID            
                breakIdx = [0, find(diff(dataTable.recordID')), length(dataTable.recordID)];
                dataTable = mat2cell(dataTable,diff(breakIdx),width(dataTable));
            end
            
            if iscell(config)
               config = config{1}; 
            end
                        
            % compute median values
            medianTableFcn = @(T)varfun(@median, T);
            A = cellfun(medianTableFcn, dataTable, 'UniformOutput',false);
            medianData = vertcat(A{:});
            
            % extract median dark data
            medianDarkData = medianData(medianData.median_recordID == 999, :);
            
            if isempty(medianDarkData)
               error('No spectrometer dark measurements found in file. Dark data is required for processing'); 
            end
            
            % extract light data
            lightData = vertcat(dataTable{:});
            lightData = lightData(lightData.recordID ~= 999, :);
            
            % remove saturated pixels
            lightData.sigPix(lightData.sigPix>64000) = NaN;               
            lightData.refPix(lightData.refPix>64000) = NaN;               

            AllDarkRefCorrected = nan(height(lightData),length(cal.wl));
            for n = 1:height(lightData)
               
                % find closest dark measurement in time
                [~,closestIndex] = min(abs(lightData.date(n) - medianDarkData.median_date));
                            
                % subtract dark
                sig_darkCorr = lightData.sigPix(n,:) - medianDarkData.median_sigPix(closestIndex,:);
                ref_darkCorr = lightData.refPix(n,:) - medianDarkData.median_refPix(closestIndex,:);
                
                % compute sig/ref
                AllDarkRefCorrected(n,:) = interp1(config.sigWls(:)', sig_darkCorr, cal.wl(:)') ./ ...
                                           interp1(config.refWls(:)', ref_darkCorr, cal.wl(:)');    
            end
            
            % save data in structure with names according to filter name in
            % firmware
            lightDataIDs = unique(lightData.recordID);
            for n = 1:length(lightDataIDs)
                measName = config.measName(config.measID == lightDataIDs(n));
                data = AllDarkRefCorrected(lightData.recordID == lightDataIDs(n), :);
                eval(['DarkRefCorrected.' char(measName) ' = data;']);
            end
            
        end
        
        %% Computes Theoretical Absorption For Pure Water
        
        function aW_TS = Get_IOCCG_aW(wls,T,S)
            
            aW_IOCCG.wl = [250;255;260;265;270;275;280;285;290;295;300;305;310;315;320;325;330;335;340;345;350;355;360;365;370;375;380;385;390;395;400;405;410;415;420;425;430;435;440;445;450;455;460;465;470;475;480;485;490;495;500;505;510;515;520;525;530;535;540;545;550;555;560;565;570;575;580;585;590;595;600;605;610;615;620;625;630;635;640;645;650;655;660;665;670;675;680;685;690;695;700;705;710;715;720;725;730;735;740;745;750;755;760;765;770;775;780;785;790;795;800;805;810;815;820;825;830;835;840;845;850;855;860;865;870;875;880;885;890;895;900];
            aW_IOCCG.aW = [0.0450000000000000;0.0392000000000000;0.0344000000000000;0.0303000000000000;0.0269000000000000;0.0240000000000000;0.0216000000000000;0.0194000000000000;0.0176000000000000;0.0160000000000000;0.0147000000000000;0.0134000000000000;0.0124000000000000;0.0114000000000000;0.0106000000000000;0.00980000000000000;0.00920000000000000;0.00850000000000000;0.00800000000000000;0.00750000000000000;0.00710000000000000;0.00680000000000000;0.00660000000000000;0.00630000000000000;0.00600000000000000;0.00560000000000000;0.00520000000000000;0.00500000000000000;0.00480000000000000;0.00470000000000000;0.00460000000000000;0.00460000000000000;0.00460000000000000;0.00460000000000000;0.00454000000000000;0.00478000000000000;0.00495000000000000;0.00530000000000000;0.00635000000000000;0.00751000000000000;0.00922000000000000;0.00962000000000000;0.00979000000000000;0.0101100000000000;0.0106000000000000;0.0114000000000000;0.0127000000000000;0.0136000000000000;0.0150000000000000;0.0173000000000000;0.0204000000000000;0.0256000000000000;0.0325000000000000;0.0396000000000000;0.0409000000000000;0.0417000000000000;0.0434000000000000;0.0452000000000000;0.0474000000000000;0.0511000000000000;0.0565000000000000;0.0596000000000000;0.0619000000000000;0.0642000000000000;0.0695000000000000;0.0772000000000000;0.0896000000000000;0.110000000000000;0.135100000000000;0.167200000000000;0.222400000000000;0.257700000000000;0.264400000000000;0.267800000000000;0.275500000000000;0.283400000000000;0.291600000000000;0.301200000000000;0.310800000000000;0.325000000000000;0.340000000000000;0.371000000000000;0.410000000000000;0.429000000000000;0.439000000000000;0.448000000000000;0.465000000000000;0.486000000000000;0.516000000000000;0.559000000000000;0.624000000000000;0.704000000000000;0.827000000000000;1.00700000000000;1.23100000000000;1.48900000000000;1.97000000000000;2.51000000000000;2.78000000000000;2.83000000000000;2.85000000000000;2.88000000000000;2.86000000000000;2.86000000000000;2.82000000000000;2.76000000000000;2.69000000000000;2.59000000000000;2.47000000000000;2.36000000000000;2.25000000000000;2.20000000000000;2.19000000000000;2.23000000000000;2.34000000000000;2.61000000000000;3.22000000000000;3.72000000000000;3.94000000000000;4.09000000000000;4.20000000000000;4.32000000000000;4.60000000000000;4.60000000000000;4.77000000000000;5.01000000000000;5.28000000000000;5.57000000000000;5.85000000000000;6.13000000000000;6.40000000000000];
            aW_IOCCG.psiT = 10.^-4 .* [3;3;2;2;2;2;2;1;1;1;1;1;1;1;1;0;0;0;1;0;-1;-1;0;0;-0.700000000000000;-1;0;0;0;-0.200000000000000;0.100000000000000;0.100000000000000;0;0.200000000000000;0;-0.100000000000000;-0.100000000000000;0;0;0.100000000000000;0.200000000000000;0.100000000000000;0.100000000000000;0;-0.100000000000000;-0.100000000000000;0;-0.100000000000000;-0.100000000000000;0;0.100000000000000;0.300000000000000;0.800000000000000;1.20000000000000;1.10000000000000;0.700000000000000;0.400000000000000;0.100000000000000;0;-0.100000000000000;0;-0.200000000000000;-0.400000000000000;-0.600000000000000;-0.700000000000000;-0.600000000000000;0;1.20000000000000;2.50000000000000;4.50000000000000;7.90000000000000;10.3000000000000;9.50000000000000;7.20000000000000;5.40000000000000;3;0.900000000000000;-0.500000000000000;-2;-3;-3;-2.20000000000000;0.800000000000000;0.900000000000000;-0.400000000000000;-2.10000000000000;-4;-4;-4.30000000000000;-4.10000000000000;-2;1.70000000000000;11;27.3000000000000;46.3000000000000;65.8000000000000;98.3000000000000;148.500000000000;161;137.200000000000;105;74.4000000000000;44.7000000000000;18.6000000000000;-4.40000000000000;-24.5000000000000;-40.4000000000000;-52.1000000000000;-59.4000000000000;-62;-60;-52;-38.4000000000000;-20;0;33;101;153;145;115;83;49;27;-0.800000000000000;-46;-63;-78;-87;-90;-85;-70];
            aW_IOCCG.psiS = 10.^-4 .* [0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.430000000000000;0.370000000000000;0.360000000000000;0.340000000000000;0.320000000000000;0.280000000000000;0.260000000000000;0.250000000000000;0.220000000000000;0.190000000000000;0.170000000000000;0.160000000000000;0.140000000000000;0.130000000000000;0.110000000000000;0.0900000000000000;0.0800000000000000;0.0600000000000000;0.0600000000000000;0.0500000000000000;0.0500000000000000;0.0400000000000000;0.0200000000000000;0.0800000000000000;0.130000000000000;0.140000000000000;0.150000000000000;0.150000000000000;0.150000000000000;0.130000000000000;0.130000000000000;0.160000000000000;0.160000000000000;0.150000000000000;0.130000000000000;0.100000000000000;0.0400000000000000;0.0100000000000000;0.0300000000000000;-0.0200000000000000;-0.160000000000000;0.430000000000000;0.750000000000000;0.830000000000000;0.840000000000000;0.800000000000000;0.770000000000000;0.730000000000000;0.700000000000000;0.660000000000000;0.600000000000000;0.340000000000000;0.410000000000000;0.630000000000000;0.620000000000000;0.460000000000000;0.250000000000000;-0.0200000000000000;-0.340000000000000;-0.700000000000000;-1.16000000000000;-1.40000000000000;-1.80000000000000;-1.90000000000000;-1.80000000000000;-2.10000000000000;-4.40000000000000;-3.70000000000000;1.80000000000000;4.70000000000000;6.50000000000000;6.80000000000000;6.60000000000000;6;5.30000000000000;4.50000000000000;3.60000000000000;2.30000000000000;1.10000000000000;-0.100000000000000;-1.30000000000000;-2.80000000000000;-4.40000000000000;-5.20000000000000;-6.20000000000000;-9;-13;-8;-2;0;0;0;-2;-6;-8;-12;-16;-20;-22;-23;-25];
            
            if isempty(wls)
                wls =  aW_IOCCG.wl;
            end
            
            aW = interp1(aW_IOCCG.wl, aW_IOCCG.aW, wls)';
            psiT = interp1(aW_IOCCG.wl, aW_IOCCG.psiT, wls)';
            psiS = interp1(aW_IOCCG.wl, aW_IOCCG.psiS, wls)';
            aW_TS = aW + (T - 22).*psiT + (S - 0).*psiS; % for  temperature, 22 is the reference; for salinity, 0 is the reference
            
        end
        
        %% Computes Reflectivity of the Sphere Given Two Solutions with Known Absorption
        
        function rho = ComputeRho(r, r_0, a_A_known, a_B_known, A_hypera, B_hypera)
            
            a_A = a_A_known;
            a_B = a_B_known;
            T_AB = A_hypera./B_hypera;
            T_AB = smoothdata(T_AB,2,'sgolay',15);
            
            % Röttgers et al. 2005 - Equation 10
            rho = (T_AB .* exp(-a_B .* r_0) .* Hypera_lib.Ps(a_B,r) - exp(-a_A .* r_0) .* Hypera_lib.Ps(a_A,r)) ...
                ./ (T_AB .* exp(-a_B .* r_0) .* Hypera_lib.Ps(a_A,r) .* Hypera_lib.Ps(a_B,r) - exp(-a_A .* r_0) .* Hypera_lib.Ps(a_B,r) .* Hypera_lib.Ps(a_A,r));
            
        end
        
        %% Computes Absorption From Hyper-a Data Given a Known Reference, Rho, and Sample
        
        function a_tot_hypera = ComputeAbsorption(cal, aW_ref, hypera_pureWater, hypera_sample)
            
            r = cal.r;
            r_0 = cal.r_0;
            rho = cal.rho;
            
            T_AB = hypera_sample./median(hypera_pureWater,1);
            T_AB = smoothdata(T_AB,2,'sgolay',15);
            
            a_tot_hypera = nan(size(T_AB));
            for i_wl = 1:length(aW_ref)
                
                T_AB_guess = @(a_guess) ...
                    exp(-r_0 .* (a_guess - aW_ref(i_wl))) ...
                    .* ((1 - rho(i_wl) .* Hypera_lib.Ps(aW_ref(i_wl),r)) ./ (1 - rho(i_wl) .* Hypera_lib.Ps(a_guess,r)) ...
                    .* Hypera_lib.Ps(a_guess,r) ./ Hypera_lib.Ps(aW_ref(i_wl),r));
                
                for i_meas = 1:height(T_AB)
                    % minimize absolute percent error
                    ToMinimize = @(a_guess) abs(T_AB_guess(a_guess) - T_AB(i_meas,i_wl))./T_AB(i_meas,i_wl);
                    a_tot_hypera(i_meas,i_wl) = fminsearch(ToMinimize, aW_ref(i_wl));
                    
                end
            end
        end
        
        %% Probablity Function
        
        function Ps_x = Ps(a,r)
            
            % Röttgers et al. 2005 - Equation 5
            Ps_x = (1 - exp(-2. * a .* r) .* (2 .* a .* r + 1)) ...
                ./ (2 .* a.^2 .* r.^2);
        end
        
    end
end