% Hypera_CaptureSpectrum collects spectral data from Hyper-a instrument in real time.
% 
% Usage Example:
%   [config, dataTable] = HyperA_CaptureSpectrum(serialPort, filtPos, numMeas)  
%   OR
%   [config, dataTable] = HyperA_CaptureSpectrum(serialPort, filtPos, numMeas,
%                                                 'PlotSpectrum', true, 'SaveTo', 'test.mat')  
% Required Arguments:
%   serialPort: Hyper-A serial port object
%   filtPos:    Measurement type (1-8), see 'filtconfig' command
%   numMeas:    Number of measurements to capture
% 
% Optional Arguments:
%   PlotSpectrum:       Plot spectrums during data collection, pass figure handle as paramter. 
%                           Passing figure handle allows data from a second
%                           call to HyperA_CaptureSpectrum to be plotted on
%                           the same axis. Pass 'figure' to generate a new
%                           figure each time.
%   PlotTime:           Plot spectral time series during data collection, pass figure handle as paramter.
%   SaveTo:             Save data to specified mat file
%
% Outputs:
%   dataTable:  A table where each row corresponds to one measurement.
%               Columns in the table are named, for easy identification
%   config:     A structure containing information about how the instrument
%               was configured.
%
% Sequoia Scientific, Inc.
% v1.0 
% 2023/11/17

function [config, dataTable] = Hypera_CaptureSpectrum(serialPort, filtPos, numMeas, varargin)

%% Parse Arguments
parser = inputParser;

addRequired(parser,'serialPort', @(x) isobject(x));
addRequired(parser,'filtPos', @(x)validateattributes(x,{'numeric'}, {'>=',1,'<=',8,'scalar','positive','integer'}));
addRequired(parser,'numMeas', @(x)validateattributes(x,{'numeric'}, {'scalar','positive','integer'}));

addParameter(parser,'PlotSpectrum',false, @(x)strcmp(get(x,'type'),'figure'));
addParameter(parser,'PlotTime',false, @(x)strcmp(get(x,'type'),'figure'));
addParameter(parser,'SaveTo',[], @(x) isempty(x)||(ischar(x)&&isLegalPath(x)));

parse(parser,serialPort,filtPos, numMeas, varargin{:});
p = parser.Results;
s = p.serialPort;

%% Collect data

% ensure binary output is turned on
Hypera_SendCommand(s, 'serialout binary',1);

% get pump settings
res = Hypera_SendCommand(s, 'usepump',1);
UsePump = ~contains(res{1},'disabled');
res = Hypera_SendCommand(s, 'pumpflush',1);
PumpFlush = sscanf(res{1},'Pump flush duration: %d sec');

% get flash settings
res = Hypera_SendCommand(s, 'exp4flash sig',4);
sigFlashes = sscanf(res{4},'  Number Of Flashes: %d');
res = Hypera_SendCommand(s, 'exp4flash ref',4);
refFlashes = sscanf(res{4},'  Number Of Flashes: %d');

% initiate measurement
Hypera_SendCommand(s, ['measfilt ' num2str(p.filtPos) ' ' num2str(p.numMeas)],0);

% what for pump flushing to finish, if needed
if (UsePump && (PumpFlush > 0))
    disp(join(['    Waiting ' num2str(PumpFlush) ' seconds for pump flush ...']))
    pause(PumpFlush)
end

% read binary header
config = Hypera_ReadHeader(s);
disp(join(['    Starting' num2str(p.numMeas) 'measurements of type' config.measName(p.filtPos) '...']))

% setup plots
if isgraphics(p.PlotSpectrum)
    figure(p.PlotSpectrum)
    subplot(1,2,1), hold on, grid on, box on
    plot([min(config.sigWls) max(config.sigWls)], [intmax('uint16') intmax('uint16')], 'k--','HandleVisibility','off')
    set(gca,'XLim',[min(config.sigWls) max(config.sigWls)])
    xlabel('Wavelength (nm)')
    ylabel('Signal Pixel Value (counts)')
    legend show
    
    lineColor = get(gca,'ColorOrder');
    lineColor = lineColor(get(gca,'ColorOrderIndex'),:);
    plot(NaN,NaN,'Color',lineColor,'DisplayName',join([config.measName(p.filtPos) '; ' num2str(sigFlashes) ' flash'],''));
    
    subplot(1,2,2), hold on, grid on, box on
    plot([min(config.refWls) max(config.refWls)], [intmax('uint16') intmax('uint16')], 'k--','HandleVisibility','off')
    set(gca,'XLim',[min(config.refWls) max(config.refWls)])
    xlabel('Wavelength (nm)')
    ylabel('Reference Pixel Value (counts)')
    legend show
    
    plot(NaN,NaN,'Color',lineColor,'DisplayName',join([config.measName(p.filtPos) '; ' num2str(refFlashes) ' flash'],''));
end

if isgraphics(p.PlotTime)
    figure(p.PlotTime)
    subplot(1,2,1), hold on, grid on, box on
    plot([1 p.numMeas], [intmax('uint16') intmax('uint16')], 'k--')
    set(gca,'XLim',[1 p.numMeas])
    xlabel('Sample Number')
    ylabel('Signal Pixel Value (counts)')
    
    subplot(1,2,2), hold on, grid on, box on
    plot([1 p.numMeas], [intmax('uint16') intmax('uint16')], 'k--')
    set(gca,'XLim',[1 p.numMeas])
    xlabel('Sample Number')
    ylabel('Reference Pixel Value (counts)')
end


% preallocate table for data records
tableVars = {
    'recordID'    , 'double';
    'date'        , 'datetime';
    'inputVoltage', 'double';
    'waterTemp'   , 'double';
    'depth'       , 'double';
    'boardTemp'   , 'double';
    'boardHumid'  , 'double';
    'lampFreq'    , 'double';
    'sigAvg'      , 'double';
    'refAvg'      , 'double';
    'sigExp'      , 'double';
    'refExp'      , 'double';
    'sigNumFlash' , 'double';
    'refNumFlash' , 'double';
    'sigPix'      , 'double';
    'refPix'      , 'double';
    };

dataTable = table('Size',[p.numMeas height(tableVars)], ...
    'VariableNames', tableVars(:,1), ...
    'VariableTypes', tableVars(:,2));

% read binary data records
for n = 1:p.numMeas
    disp(join(['    ' num2str(n)]));
    
    dataTable.recordID(n)     = read(s,1,'uint16');
    dataTable.date(n)         = datetime(read(s,6,'uint8')) + calyears(1900);
    dataTable.inputVoltage(n) = read(s,1,'uint16') / 100;
    dataTable.waterTemp(n)    = read(s,1,'uint16') / 100 - 10;
    dataTable.depth(n)        = read(s,1,'uint16') / 100 - 10;
    dataTable.boardTemp(n)    = read(s,1,'uint16') / 100 - 10;
    dataTable.boardHumid(n)   = read(s,1,'uint8');
    dataTable.lampFreq(n)     = read(s,1,'uint8');
    dataTable.sigAvg(n)       = read(s,1,'uint8');
    dataTable.refAvg(n)       = read(s,1,'uint8');
    dataTable.sigExp(n)       = read(s,1,'uint32');
    dataTable.refExp(n)       = read(s,1,'uint32');
    dataTable.sigNumFlash(n)  = read(s,1,'uint16');
    dataTable.refNumFlash(n)  = read(s,1,'uint16');
    
    dataTable.sigPix(n,1:config.sigNumWls) = read(s,config.sigNumWls,'uint16');
    dataTable.refPix(n,1:config.refNumWls) = read(s,config.refNumWls,'uint16');
    
    
    if isgraphics(p.PlotSpectrum)
        figure(p.PlotSpectrum)
        subplot(1,2,1)
        plot(config.sigWls,dataTable.sigPix(n,:),'Color',lineColor,'HandleVisibility','off');

        subplot(1,2,2)
        plot(config.refWls,dataTable.refPix(n,:),'Color',lineColor,'HandleVisibility','off')
        drawnow;
    end
    
    if isgraphics(p.PlotTime)
        figure(p.PlotTime)
        subplot(1,2,1)
        plot(1:n,dataTable.sigPix(1:n,interp1(config.sigWls,1:config.sigNumWls,350,'nearest')),'m.-')
        plot(1:n,dataTable.sigPix(1:n,interp1(config.sigWls,1:config.sigNumWls,450,'nearest')),'b.-')
        plot(1:n,dataTable.sigPix(1:n,interp1(config.sigWls,1:config.sigNumWls,550,'nearest')),'g.-')
        plot(1:n,dataTable.sigPix(1:n,interp1(config.sigWls,1:config.sigNumWls,650,'nearest')),'r.-')
        legend({'','350','450','550','650'});
        
        subplot(1,2,2)
        plot(1:n,dataTable.refPix(1:n,interp1(config.refWls,1:config.refNumWls,350,'nearest')),'m.-')
        plot(1:n,dataTable.refPix(1:n,interp1(config.refWls,1:config.refNumWls,450,'nearest')),'b.-')
        plot(1:n,dataTable.refPix(1:n,interp1(config.refWls,1:config.refNumWls,550,'nearest')),'g.-')
        plot(1:n,dataTable.refPix(1:n,interp1(config.refWls,1:config.refNumWls,650,'nearest')),'r.-')
        drawnow;
    end
    
    
end

%%  clean up
if ~isempty(p.SaveTo)
    save(p.SaveTo,'config','dataTable');
    disp(join(['    Data saved to:' p.SaveTo]))
end

flush(s)
disp('    Measurement Finished')

end

function config = Hypera_ReadHeader(s)

config.serialNumber = read(s,1,'uint16');
config.firmwareVer  = read(s,1,'uint16') / 100;
config.sigSpecSN    = read(s,1,'uint32');
config.sefSpecSN    = read(s,1,'uint32');
config.sigNumWls    = read(s,1,'uint16');
config.refNumWls    = read(s,1,'uint16');
config.configByte   = read(s,1,'uint8');
config.mainBoardRev = read(s,1,'char');
config.pumpFlushSec = read(s,1,'uint16');

if config.firmwareVer >= 1.07 
    % added in v1.07
    config.sequenceInterval_sec = read(s,1,'uint16');
	config.burstInterval_min = read(s,1,'uint16');
	config.sequencesPerBurst = read(s,1,'uint16');
    read(s,1,'uint16'); % unassigned, used to preserve memory alignment
end

for n=1:8
    config.measName(n,:) = deblank(read(s,16,'string'));
end
config.measID       = read(s,8,'uint16');
config.sigWls       = read(s,config.sigNumWls,'uint32') / 1000;
config.refWls       = read(s,config.refNumWls,'uint32') / 1000;

config.autoExposure = bitget(config.configByte,1);
config.sigLinCorr   = bitget(config.configByte,2);
config.refLinCorr   = bitget(config.configByte,3);
config.pumpEnabled  = bitget(config.configByte,4);
config.autoStart    = bitget(config.configByte,5);
config.switchStart  = bitget(config.configByte,6);
end


function bool = isLegalPath(str)
bool = true;
try
    java.io.File(str).toPath;
catch
    bool = false;
end
end



