% Hypera_Configure configures the Hyper-a instrument according to the
% various configuration options passed to the function. 
%
% Usage Example:
%   HyperA_Configure(serialPort, 'FlashHz', 50, 'nFlashSig, 2, 'nFlashRef', 2, 'AutoExposure', false)  
%
% Required Arguments:
%   serialPort: Hyper-A serial port object
% 
% Optional Arguments:
%   FlashHz:            Lamp rigger frequency (2-120 hz)
%   nFlashSig:          Number of lamp flashes to capture for sig spec
%   nFlashRef:          Number of lamp flashes to capture for ref spec
%   SigPixelRange:      Set starting and ending pix for sig spectrometer (e.g [100 2000])
%   RefPixelRange:      Set starting and ending pix for ref spectrometer
%   SigLinearityCorr:   Turn on linearity correction for sig spectrometer
%   ReflinearityCorr:   Turn on linearity correction for ref spectrometer
%   SigExposureAvg:     Set exposure averaging number for sig spectrometer
%   RefExposureAvg:     Set exposure averaging number for sig spectrometer
%   AutoExposure:       Turn on automatic exposure adjustments to bring signals within range
%   SaveDataFiles:      Turn off saving data files onboard the instrument. Will boost measurement speed.
%   UsePump:            Turn on pump during data collection
%   PumpFlush:          Run pump for specified number of seconds before starting measurement
%
% Sequoia Scientific, Inc.
% v1.0 
% 2023/11/17

function Hypera_Configure(serialPort, varargin)

%% Parse Arguments
parser = inputParser;

% instrument configuration
addRequired(parser,'serialPort', @(x) isobject(x));

addParameter(parser,'FlashHz',50, @(x)validateattributes(x,{'numeric'}, {'>=',2,'<=',120,'scalar','positive','integer'}));
addParameter(parser,'nFlashSig',2, @(x)validateattributes(x,{'numeric'}, {'scalar','positive','integer'}));
addParameter(parser,'nFlashRef',2, @(x)validateattributes(x,{'numeric'}, {'scalar','positive','integer'}));
addParameter(parser,'SigPixelRange',[458 1835], @(x)validateattributes(x,{'numeric'}, {'>=',0,'<=',2047, 'numel',2,'integer'}));
addParameter(parser,'RefPixelRange',[460 1837], @(x)validateattributes(x,{'numeric'}, {'>=',0,'<=',2047,'numel',2,'positive','integer'}));
addParameter(parser,'SigLinearityCorr',false, @(x)validateattributes(x,{'logical'}, {'scalar'}));
addParameter(parser,'ReflinearityCorr',false, @(x)validateattributes(x,{'logical'}, {'scalar'}));
addParameter(parser,'SigExposureAvg',1, @(x)validateattributes(x,{'numeric'}, {'<=',255, 'positive','integer'}));
addParameter(parser,'RefExposureAvg',1, @(x)validateattributes(x,{'numeric'}, {'<=',255, 'positive','integer'}));
addParameter(parser,'AutoExposure',false, @(x)validateattributes(x,{'logical'}, {'scalar'}));
addParameter(parser,'SaveDataFiles',true, @(x)validateattributes(x,{'logical'}, {'scalar'}));
addParameter(parser,'UsePump',false, @(x)validateattributes(x,{'logical'}, {'scalar'}));
addParameter(parser,'PumpFlush',0, @(x)validateattributes(x,{'numeric'}, {'>=',0,'scalar','integer'}));

parse(parser,serialPort,varargin{:});
p = parser.Results;
s = p.serialPort;

%% Configure Instrument
% Only set the value if it was provided as an argument to the function.
% Parameters not passed to the function are left at their current value

if (~ismember('FlashHz',parser.UsingDefaults))
    Hypera_SendCommand(s, ['lampfreq ' num2str(p.FlashHz)], 2);
end

if (~ismember('SigPixelRange',parser.UsingDefaults))
    Hypera_SendCommand(s, ['pxrange sig ' num2str(p.SigPixelRange(1)) ' ' num2str(p.SigPixelRange(2))], 2);
end

if (~ismember('SigLinearityCorr',parser.UsingDefaults))
    Hypera_SendCommand(s, ['lincorr sig ' num2str(p.SigLinearityCorr)], 1);
end

if (~ismember('SigExposureAvg',parser.UsingDefaults))
    Hypera_SendCommand(s, ['imavg sig ' num2str(p.SigExposureAvg)], 1);
end

if (~ismember('RefPixelRange',parser.UsingDefaults))
    Hypera_SendCommand(s, ['pxrange ref ' num2str(p.RefPixelRange(1)) ' ' num2str(p.RefPixelRange(2))], 2);
end

if (~ismember('ReflinearityCorr',parser.UsingDefaults))
    Hypera_SendCommand(s, ['lincorr ref ' num2str(p.ReflinearityCorr)], 1);
end

if (~ismember('RefExposureAvg',parser.UsingDefaults))
    Hypera_SendCommand(s, ['imavg ref ' num2str(p.RefExposureAvg)], 1);
end

if (~ismember('nFlashSig',parser.UsingDefaults))
    Hypera_SendCommand(s, ['exp4flash sig ' num2str(p.nFlashSig)], 6);
end

if (~ismember('nFlashRef',parser.UsingDefaults))
    Hypera_SendCommand(s, ['exp4flash ref ' num2str(p.nFlashRef)], 6);
end

if (~ismember('AutoExposure',parser.UsingDefaults))
    Hypera_SendCommand(s, ['autoexp ' num2str(p.AutoExposure)],1);
end

if (~ismember('SaveDataFiles',parser.UsingDefaults))
    Hypera_SendCommand(s, ['savedata ' num2str(p.SaveDataFiles)],1);
end

if (~ismember('UsePump',parser.UsingDefaults))
    Hypera_SendCommand(s, ['usepump ' num2str(p.UsePump)],1);
end

if (~ismember('PumpFlush',parser.UsingDefaults)) 
    Hypera_SendCommand(s, ['pumpflush ' num2str(p.PumpFlush)],1);
end


end